import pytest

from pyrsvp.rsvpserver import RsvpServerTcp, RsvpServer
from pyrsvp.rsvpclient import *
from pyrsvp.rsvptypes import *
import multiprocessing

@RsvpServer.Command("START")
def start(values: dict) -> str:
    
    print(values)
    return RsvpServer.reply(True, [RsvpString("STATUS", "PASS")])

@RsvpServer.Command("STOP")
def stop(values: dict) -> str:
    
    return RsvpServer.reply(False)

@RsvpServer.Command("ERROR")
def error(values: dict) -> str:

    return RsvpServer.reply(False, [RsvpString("ERROR", "TEST ERROR HANDLING")])

@RsvpServer.Command("NOACKNACK")
def no_ack_nack(values: dict) -> str:

    return ""

@RsvpServer.Command("MISSINGACKNACK")
def missing_ack_nack(values: dict) -> str:

    return "MISSING_ACK_NAK"

@RsvpServer.Command("NACK")
def nack(values: dict) -> str:

    return RsvpServer.reply(False)

def tcp_server():
    srv = RsvpServerTcp()
    print("Tcp Server start")
    srv.run()

@pytest.fixture(scope="module", autouse=True)
def module_start():

    p = multiprocessing.Process(target=tcp_server)
    p.start()
    yield
    p.terminate()


@pytest.fixture(scope="module")
def client() -> RsvpClientTcp:
    return RsvpClientTcp()

@pytest.fixture(scope="module")
def too_many_args() -> list[RsvpBaseType]:

    args = [
        RsvpInteger("INTEGER", 1),
        RsvpDouble("DOUBLE", 2.0),
        RsvpBool("BOOL", True),
        RsvpString("STRING", "hello world"),
        RsvpVectorDouble("DOUBLE_VEC", {"X": 1.0, "Y": 2.0, "Z": 3.0}),
        RsvpStringVector("STRING_VEC", ["Hello", "World"]),
        RsvpStringVector("STRING_VEC", ["Hello", "World"])
    ]

    return args

@pytest.fixture(scope="module")
def too_few_args() -> list[RsvpBaseType]:

    args = [
        RsvpInteger("INTEGER", 1),
        RsvpDouble("DOUBLE", 2.0),
        RsvpBool("BOOL", True),
        RsvpString("STRING", "hello world"),
        RsvpVectorDouble("DOUBLE_VEC", {"X": 1.0, "Y": 2.0, "Z": 3.0})
    ]

    return args

class TestTcpClient:

    def test_bad_reply(self, client : RsvpClientTcp):

        with pytest.raises(RsvpMalformedReply) as ex:
            client.send_command("NOACKNACK")
            client.send_command("MISSINGACKNACK")
        

    def test_nack(self, client : RsvpClientTcp):

        with pytest.raises(RsvpNotAck):
            client.send_command("NACK")

    def test_error_handling(self, client : RsvpClientTcp):

        with pytest.raises(RsvpNotAck):
            client.send_command("ERROR")

    def test_wrong_arg_type(self, client: RsvpClientTcp):

        with pytest.raises(RsvpWrongTypeException):
            client.send_command("WRONG_TYPE", args=[RsvpInteger("INT", "Hello")])

    def test_wrong_reply(self, client: RsvpClientTcp):

        with pytest.raises(RsvpNotAck):
            client.send_command("STOP")

    def test_reply_string(self, client: RsvpClientTcp):

        reply = client.send_command("START")
        assert reply["STATUS"] == "PASS"